#include "rtdef.h"
#include <am.h>
#include <klib.h>
#include <rtthread.h>

static Context* ev_handler(Event e, Context *c) {
  switch (e.event) {
    case EVENT_YIELD: {
      // Context Switch
      if (c->gpr[11] == 0) {
        Context **to = (void *)c->gpr[10];
        *c = **to;
      } else {
        Context **from = (void *)c->gpr[10];
        Context **to = (void *)c->gpr[11];
        *from = c;
        *(Context **)(c->gpr[10]) = c;
        *c = **to;
      }
      break;
    }

    default: printf("Unhandled event ID = %d\n", e.event); assert(0);
  }
  return c;
}

void __am_cte_init() {
  cte_init(ev_handler);
}

void rt_hw_context_switch_to(rt_ubase_t to) {
  asm volatile("li a1, 0");
  yield();
}

void rt_hw_context_switch(rt_ubase_t from, rt_ubase_t to) {
  yield();
}

void rt_hw_context_switch_interrupt(void *context, rt_ubase_t from, rt_ubase_t to, struct rt_thread *to_thread) {
  assert(0);
}

typedef struct {
  void (*tentry)(void *);
  void *parameter;
  void (*texit)(void);
} tenter_parameters;

__attribute__((noreturn)) void tenter_wrapper(void *p) {
  tenter_parameters *pp = p;
  pp->tentry(pp->parameter);
  pp->texit();
  assert(0);
}

rt_uint8_t *rt_hw_stack_init(void *tentry, void *parameter, rt_uint8_t *stack_addr, void *texit) {
  stack_addr = stack_addr - (uintptr_t)stack_addr % 4;
  tenter_parameters *p_params = (tenter_parameters*)(stack_addr - sizeof(tenter_parameters));
  p_params->tentry = tentry;
  p_params->parameter = parameter;
  p_params->texit = texit;
  void *sp = kcontext((Area) {(void *)(stack_addr - sizeof(tenter_parameters) - sizeof(Context) - 4), (stack_addr - sizeof(tenter_parameters))}, (void *)tenter_wrapper, (void *)p_params);
  return sp;
}
